"""Send email notifications for all tasks that have failed in the last hour.

This task must be scheduled to run every hour to prevent missing out on notifications.
"""

from datetime import UTC, datetime

from orchestrator.domain import SubscriptionModel
from orchestrator.targets import Target
from orchestrator.types import State
from orchestrator.workflow import StepList, done, init, step, workflow

from gso.services.mailer import send_mail
from gso.services.subscriptions import get_failed_tasks, get_subscription_id_by_process_id
from gso.settings import load_oss_params


@step("Gather all tasks that recently failed")
def gather_failed_tasks() -> State:
    """Gather all tasks from the last hour that have failed."""
    current_time = datetime.now(UTC)

    failed_tasks = list(filter(lambda task: (current_time - task.last_modified_at).hour <= 1, get_failed_tasks()))

    return {"failed_tasks": failed_tasks}


@step("Send notification emails for all failed tasks")
def send_email_notifications(state: State) -> None:
    """Send out an email notification for every task that has failed in the last hour."""
    base_url = load_oss_params().GENERAL.public_hostname
    for failure in state["failed_tasks"]:
        subscription_id = get_subscription_id_by_process_id(failure["process_id"])
        failed_subscription = SubscriptionModel.from_subscription(subscription_id)
        failed_task_url = f"{base_url}/workflows/{failure["process_id"]}"

        send_mail(
            "GAP - A task has failed!",
            f"""A {failed_subscription.product.name} validation task has failed to run to completion.

Product name: {failed_subscription.product.name}
Description: {failed_subscription.description}
The step "{failure["last_step"]}" failed for the following reason: "{failure["failed_reason"]}".

Please inspect the full workflow at the following link: {failed_task_url}.
""",
        )


@workflow("Send email notifications for all failed tasks", target=Target.SYSTEM)
def task_send_email_notifications() -> StepList:
    """Gather all failed tasks from the last hour, and send email notifications."""
    return init >> gather_failed_tasks >> send_email_notifications >> done
